"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
// Light-weight
const request = require("request-promise-native");
class API {
    constructor(baseAPIURL, token) {
        this.baseAPIURL = baseAPIURL;
        this.token = token;
    }
    getUserInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = yield this.request("config").catch(() => {
                throw new Error("Unable to fetch user info.");
            });
            const npmrc = yield this.parseNpmrcResponse(data);
            data.npmrc = npmrc.replace("_authToken=TOKEN", `_authToken=${this.token}`);
            return data;
        });
    }
    getPackagesMeta(packages) {
        return this.request("packages-meta", { packageNames: packages }).catch(() => {
            throw new Error("Unable to fetch packages metadata.");
        });
    }
    parseNpmrcResponse(response) {
        return new Promise((resolve, reject) => {
            if (response.npmrc === undefined) {
                reject();
            }
            else {
                resolve(Buffer.from(response.npmrc, "base64").toString());
            }
        });
    }
    request(endpoint, body) {
        return __awaiter(this, void 0, void 0, function* () {
            const { method, path } = API.endpoints[endpoint];
            const uri = `${this.baseAPIURL}${path}`;
            return request({ method, uri, headers: { Authorization: `Token ${this.token}` }, body, json: true });
        });
    }
}
API.endpoints = {
    "packages-meta": { path: "/studio/packages/meta/get-many", method: "POST" },
    config: { path: "/studio/me", method: "GET" },
};
exports.API = API;
