"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const semver = require("semver");
const chalk_1 = require("chalk");
const project_1 = require("../../../project");
const interface_1 = require("../../interface");
const validators = require("./validators");
exports.publish = (argv, config) => __awaiter(this, void 0, void 0, function* () {
    // Validate inputs and package.json configuration before continuing
    validators.validatePublishInput();
    const project = yield project_1.Project.load(config);
    const { packageJson: pkg } = project;
    if (!validators.validatePackage(pkg))
        return;
    const isPublic = argv.public !== undefined;
    if (!isPublic && !(yield project.canBePublishedPrivately)) {
        throw new Error("Package cannot be published privately.");
    }
    const pkgName = yield project.generateNewPackageName(!isPublic);
    const nextVersion = yield getNextVersion(project, argv.major !== undefined ? "major" : "minor");
    yield project.install();
    // Confirm the user wants to publish the version
    // (unless the --yes flag is passed in)
    if (!config.disablePrompts) {
        yield promptBeforePublishing(pkgName, nextVersion, isPublic);
    }
    yield project.publish(nextVersion, pkgName, undefined, {}, isPublic ? "public" : "restricted");
});
const getNextVersion = (project, bump) => __awaiter(this, void 0, void 0, function* () {
    // Get last published version
    const latestPublishedVersion = yield project.getLatestPublishedVersion();
    // Get package version
    const { packageJson: { version: pkgVersion }, } = project;
    // If the package version is higher than the last published version, use the package version. Otherwise bump the last published version.
    let version;
    if (pkgVersion && semver.gt(pkgVersion, latestPublishedVersion)) {
        version = pkgVersion;
    }
    else {
        version = semver.inc(latestPublishedVersion, bump);
    }
    validators.validateVersion(version, "Cannot update package version.");
    return version;
});
const promptBeforePublishing = (name, version, isPublic) => __awaiter(this, void 0, void 0, function* () {
    let promptText = `Ready to publish `;
    promptText += chalk_1.default `{bold ${name}} v{bold ${version}} `;
    if (isPublic) {
        promptText += chalk_1.default `({yellowBright public store}). `;
    }
    else {
        promptText += chalk_1.default `({greenBright private store}). `;
    }
    promptText += `Continue? `;
    const answer = yield interface_1.prompt(promptText);
    if (!answer.match(/^(y|yes)/i)) {
        process.stdout.write("Aborting…");
        // Terminate the process cleanly
        process.exit();
    }
});
