"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const semver = require("semver");
const handleError_1 = require("../../handleError");
// our convention says names must be @<registry>/<scope>.<name>
// only letters, underscores, dashes, and dots allowed
// scopes and package names cannot start with dot or underscore
// names cannot be longer than 214 chars (including scope)
// https://docs.npmjs.com/files/package.json#name
const nameRegEx = /^@[a-z\-\d]+[a-z_\-.\d]*\/[a-z\-\d]+[a-z_\-\d]*\.[a-z_\-\d.]+$/;
const nameLengthRestriction = 214;
exports.validatePackage = (pkg) => {
    if (!pkg.name || !pkg.version || !pkg.framer || !pkg.framer.displayName) {
        handleError_1.handleError("Missing package info.\nPlease publish the package through the store first.");
    }
    exports.validateName(pkg.name);
    exports.validateVersion(pkg.version, "Invalid package version.");
    return true;
};
exports.validatePublishInput = () => {
    if (!process.env.FRAMER_TOKEN) {
        handleError_1.handleError("Missing FRAMER_TOKEN environment variable.");
    }
};
exports.validateName = (name) => {
    if (!name || !nameRegEx.test(name) || name.length > nameLengthRestriction) {
        handleError_1.handleError("Invalid package name.");
    }
};
exports.validateVersion = (version, errorMessage) => {
    if (!version || !semver.valid(version)) {
        handleError_1.handleError(errorMessage);
        return;
    }
};
