#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const interface_1 = require("./interface");
const handleError_1 = require("./handleError");
const buildConfig_1 = require("./buildConfig");
const commands = require("./commands");
const main = () => __awaiter(this, void 0, void 0, function* () {
    interface_1.setProcessTitle("Framer Project");
    // Command is treated specially when parsing argv
    const { command, argv } = interface_1.parseArgv();
    // Terminate early if command does not exist
    if (!(command in commands)) {
        interface_1.printUsage();
        // Terminate with an error code
        process.exit(1);
    }
    // Build the configuration needed by the project
    const config = buildConfig_1.buildConfig(argv);
    // Errors will be handled by the `main` Promise catch
    return commands[command](argv, config);
});
// Terminate the process after command completes
main().then(() => process.exit(), e => handleError_1.handleError(e.message || e));
