"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const minimist = require("minimist");
const chalk_1 = require("chalk");
const project_1 = require("./project");
const server_1 = require("./server");
process.title = "Framer Project";
/*
CLI
-------------------------------------------------------------------------------
Run an HTTP server with auto reload. Use --staticPath to serve the
specified path under /static/.

Use --no-live-sharing to disable Live Sharing when serving.
-------------------------------------------------------------------------------
Options:
--api=<url>                 API endpoint to use
--author=<name>             Set author if there is none [HACK!]
--cachePath=<path>          Use a custom cache path
--registry=<domain>         Registry to use (for publishing and @framer/*)
--yarnLinkFolder=<path>     The path to the yarn link folder that should be used to link packages
*/
function serve(project, argv) {
    return __awaiter(this, void 0, void 0, function* () {
        const chosenPort = yield project.serve(portRange(argv.port || argv.p));
        process.title += ` ${chosenPort}`;
        yield project.install();
        project.watch();
        return true; // don’t exit
    });
}
const main = () => __awaiter(this, void 0, void 0, function* () {
    const argv = minimist(process.argv.slice(2));
    setupCocoaProcessHandler();
    let { registryRefreshInterval } = argv;
    if (typeof registryRefreshInterval !== "number") {
        registryRefreshInterval = undefined;
    }
    const config = {
        projectPath: argv._[0],
        staticPath: argv.staticPath,
        cachePath: argv.cachePath || "/tmp/framer",
        yarnLinkFolder: argv.yarnLinkFolder || "/tmp/framer-yarn-link-folder",
        token: argv.token,
        api: argv.api,
        author: argv.author,
        registry: argv.registry,
        registryRefreshInterval,
        proxyURL: argv.proxyURL,
        proxyDocumentURL: argv.proxyDocumentURL,
        proxyImageBaseURL: argv.proxyImageBaseURL,
        disableLiveSharing: argv["live-sharing"] !== undefined && !argv["live-sharing"],
        disablePrompts: argv.yes || false,
    };
    if (!config.projectPath) {
        // Directory not available yet – the project will be created later.
        const server = new server_1.Server(config);
        const chosenPort = yield server.start(portRange(argv.port || argv.p));
        process.title += ` ${chosenPort}`;
        return; // don’t exit
    }
    const project = yield project_1.Project.load(config);
    yield serve(project, argv);
});
main().catch(e => error(e.message));
// Because Raven is slow, install it after the server started (we’re not using the Middleware)
require("raven")
    .config("https://05cdf9fe436f4b058c893701e0a577c8@sentry.io/1253080")
    .install();
function error(message) {
    // tslint:disable-next-line:no-console
    console.log(chalk_1.default `\n{redBright ERROR: ${message}}`);
    // Terminate with failure code
    return process.exit(1);
}
function portRange(base = 4567, count = 100) {
    return [base, base + count];
}
function setupCocoaProcessHandler() {
    // The macOS application uses a stdio connection as a method of controlling the child cli process
    // when the application closes the stdin connection it expects the child process to exit. This is
    // to work around the unreliable behaviour of NSTask when attempting to force termination.
    // Begin reading from stdin so the process does not exit.
    process.stdin.resume();
    process.stdin.on("end", () => {
        process.exit();
    });
}
